import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.*;
import javax.swing.border.BevelBorder;

public class MainFrame extends JFrame
{
	private final int sizeX = 500; //X width
	private final int sizeY = 600; //Y height
	private final String radioNames[] = {"Binary","Octal","Decimal",
	        "Hexadecimal","ASCII"};
	private JRadioButton radio[];
	private JTextField inputText;
	private JTextArea outputText;
	private JLabel message;
	private ButtonGroup group;
	private String currentRadio;
	private AudioClip successSFX1,successSFX2,errorSFX1,errorSFX2,byeSFX;
	
	public void transformFrame()
	{	
		setTitle("DATA REPRESENTATION TOOL V1.0");
		setLocation(75,75);
		setResizable(false);
		setSize(sizeX,sizeY);
		
		//AUDIO Setup
	    try
	    {
	        successSFX1 = Applet.newAudioClip((new File("sfx/perfect.wav")).toURL());
	        successSFX2 = Applet.newAudioClip((new File("sfx/excellent.wav")).toURL());
	        errorSFX1   = Applet.newAudioClip((new File("sfx/nooo.wav")).toURL());
	        errorSFX2   = Applet.newAudioClip((new File("sfx/oops.wav")).toURL());
	        byeSFX      = Applet.newAudioClip((new File("sfx/byebye.wav")).toURL());
		}
	    catch (Exception e){ System.err.println("SXF FILES NOT FOUND OR CAN NOT BE OPENED!!");}
	
		// Get content pane and set main Layout
		Container container = getContentPane();
		FlowLayout flayout = new FlowLayout(0,5,10);
		flayout.setAlignment(FlowLayout.LEFT);
		container.setLayout(flayout);
		
		//Title Panel and it's Label
		JPanel titlePanel = new JPanel();
		JLabel titleLabel = new JLabel(new ImageIcon("gfx/title.jpg"));
		titlePanel.add(titleLabel);
		container.add(titlePanel);
		
		//InputType Panel and it's Label(input type label, radio button group)
		JPanel inputTypePanel = new JPanel();
		JLabel inputTypeLabel = new JLabel(" INPUT TYPE : ");
		inputTypeLabel.setBorder( new BevelBorder(BevelBorder.RAISED));
		inputTypePanel.add(inputTypeLabel);
		
		//Radio Buttons Panel and buttons
		JPanel radioPanel = new JPanel();
		radioPanel.setLayout(new GridLayout(2,radioNames.length));
		radioPanel.setBorder( new BevelBorder(BevelBorder.LOWERED));
		group = new ButtonGroup();
		radio = new JRadioButton[radioNames.length];
		for(int i=0; i<radioNames.length; i++)
		{
		    radio[i] = new JRadioButton(radioNames[i]);
		    radio[i].addActionListener(
			        new ActionListener()
			        {
	                    public void actionPerformed(ActionEvent event) 
	                    {
	                        currentRadio = event.getActionCommand();
	                        organizeComment();
	                    }
			        }
			);
		    radio[0].setSelected(true); currentRadio = "Binary";
		    group.add(radio[i]);
		    radioPanel.add(radio[i]);
		}
		inputTypePanel.add(radioPanel);
		container.add(inputTypePanel);
		
		//1 st Seperator Panel and it's Label
		JPanel seperatorPanel  = new JPanel();
		JLabel seperatorLabel  = new JLabel(new ImageIcon("gfx/seperator.jpg"));
		seperatorPanel.add(seperatorLabel);
		container.add(seperatorPanel);

		
		//Comment & Error Panel and its Label
		JPanel comPanel = new JPanel();
		JLabel comLabel = new JLabel("Comment : ");
		message  = new JLabel();//Error and Comment messages are written here
	    organizeComment();
		comPanel.add(comLabel);
		comPanel.add(message);
		container.add(comPanel);
		
		//Input Text Panel(input label, text field, transform button)
		JPanel inputPanel = new JPanel();
		JLabel inputLabel = new JLabel(" INPUT : ");
		inputLabel.setBorder( new BevelBorder(BevelBorder.RAISED));
		inputPanel.add(inputLabel);
		//Text Field
		inputText = new JTextField("Enter Your Input Here...",33);
		inputText.setFont(new Font("Monospaced",Font.BOLD,15));
		inputText.addActionListener(
		        new ActionListener()
		        {
                    public void actionPerformed(ActionEvent event) 
                    {
                       organizeInput(event.getActionCommand());
                    }
		        }
		);
		inputPanel.add(inputText);
		//Transform button
		JButton transButton = new JButton(new ImageIcon("gfx/transform.gif"));
		transButton.addActionListener(
		        new ActionListener()
		        {
                    public void actionPerformed(ActionEvent event) 
                    {
                        organizeInput(inputText.getText());
                    }
		        }
		);
		inputPanel.add(transButton);
		container.add(inputPanel);//Add input panel to container
		
		//2nd Seperator Panel and it's Label
		JPanel seperatorPanel2  = new JPanel();
		JLabel seperatorLabel2  = new JLabel(new ImageIcon("gfx/seperator.jpg"));
		seperatorPanel2.add(seperatorLabel2);
		container.add(seperatorPanel2);
		
		//Output Panel and it's Label
		JPanel outputPanel = new JPanel();
		JLabel outputLabel = new JLabel(" OUTPUT : ");
		outputLabel.setBorder( new BevelBorder(BevelBorder.RAISED));
		outputPanel.add(outputLabel);
		container.add(outputPanel);
		
		//Output Panel2 results are written Here
		JPanel outputPanel2 = new JPanel();
		outputText = new JTextArea();
		outputText.setFont(new Font("Monospaced",Font.BOLD,15));
		outputText.setEditable(false);
		outputPanel2.add(outputText);
		container.add(outputPanel2);
		
		// create menu bar
		JMenuBar bar = new JMenuBar();
		setJMenuBar(bar);
		
		// create Main menu(File) menu 
		JMenu fil = new JMenu("File");
		fil.setMnemonic('F');
		bar.add(fil);
		
		// create File menu item
		JMenuItem  trans = new JMenuItem("Transformations");
		trans.setMnemonic('T');
		fil.add(trans);
		trans.addActionListener( //Action Listener for Transformation
			new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
				    dispose();
				    MainFrame m = new MainFrame();
			        m.transformFrame();
			        m.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				}
			}
		);
		
		// create Exit menu item
		JMenuItem exit = new JMenuItem("Exit");
		exit.setMnemonic('E');
		fil.add(exit);	
		exit.addActionListener( //Action Listener for Exit
			new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
				    byeSFX.play();
				    JOptionPane.showMessageDialog(MainFrame.this,
				            "BYE BYE ! ! !","Bye Bye. . .",JOptionPane.PLAIN_MESSAGE);
					System.exit(0);
				}
			}
		);
		
		// create About Menu
		JMenu about = new JMenu("About");
		about.setMnemonic('A');
		bar.add(about);
			
		// create Help menu item
		JMenuItem help = new JMenuItem("Help");
		help.setMnemonic('H');
		about.add(help);
		help.addActionListener( //Action Listener for Help
			new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{				
				    HelpCredits h = new HelpCredits();
				    h.setLocation(125,100);
			        h.help();
			        h.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				}
			}
		);
		
		//create Credits menu item
		JMenuItem credits = new JMenuItem("Credits");
		credits.setMnemonic('C');
		about.add(credits);
		credits.addActionListener( //Action Listener for Credits
			new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
				    HelpCredits h = new HelpCredits();
				    h.setLocation(125,100);
			        h.credits();
			        h.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				}
			}
		);
		setVisible(true);
	}//transformFrame()
	
	//Organize action listeners of textfield and buttons
	private void organizeInput(String input)
	{
	    TransCalculator calc = new TransCalculator();
	    String output = "";
	    int bin[] = new int[32];//Binary Number, main data type used by the functions
	    
	    switch (currentRadio.charAt(0))
	    {
	    	case 'B':
	    	    try{
	    	        if(input.length()>32) throw new Exception();
	    	        
	    	        for(int i=0; i<input.length(); i++)
	    	        {
	    	            int tempBit = Integer.parseInt(input.charAt(i)+"");
	    	            if(!(tempBit==0 || tempBit ==1))
	    	                throw new Exception();
	    
	    	            bin[input.length()-i-1] = tempBit;
	    	        }
	    	    }catch(Exception e){
	    	        errorSFX1.play();
	    	        message.setForeground(Color.RED);
	    	        message.setText("NOT BINARY OR BINARY OUT OF BOUNDS(max 32-BIT) ! !");
                    return;
                }
	    	    
	    	    successSFX1.play();
	    	    message.setForeground(Color.GREEN);
	    	    message.setText("BINARY TRANSFORMED");
	    	    break;
	    	    
	    	case 'O':
	    	    try{
	    	        if(input.length()>10) throw new Exception();
	    	        if(input.contains("9")) throw new Exception();
	    	        if(input.contains("8")) throw new Exception();
	    	        if(input.contains("-")) throw new Exception();
	    	        Integer.parseInt(input);//To catch non integer characters 
	    	    }
	    	    catch(Exception e){
	    	        errorSFX2.play();
	    	        message.setForeground(Color.RED);
	    	        message.setText("NOT OCTAL NUMBER OR IT'S OUT OF BOUNDS(max 10-DIGIT) ! !");
	    	        return;
                }
	    	    
	    	    bin = calc.octalToBinary(input);
	    	    successSFX2.play();
	    	    message.setForeground(Color.GREEN);
	    	    message.setText("OCTAL NUMBER TRANSFORMED");
	    	    break;
	    	    
	    	case 'D':
	    	    int tempInt;//Temporary integer
	    	    try{
	    	        tempInt = Integer.parseInt(input);
	    	    }
	    	    catch (Exception e){
	    	        errorSFX1.play();
	    	        message.setForeground(Color.RED);
	    	        message.setText("NOT INTEGER OR INTEGER OUT OF BOUNDS ! !");
	    	        return;
                }
	    	    
	    	    bin = calc.integerToBinary(tempInt);
	    	    successSFX1.play();
	    	    message.setForeground(Color.GREEN);
	    	    message.setText("INTEGER TRANSFORMED");
	    	    break;
	    	    
	    	case 'H':
	    	    input = input.toUpperCase();
	    	    try{
	    	        if(input.length()>8) throw new Exception();
	    	        for(int i=0; i<input.length(); i++)
	    	        {
	    	            if(input.charAt(i) == 'A' || input.charAt(i) == 'B' || 
	    	               input.charAt(i) == 'C' || input.charAt(i) == 'D' ||
	    	               input.charAt(i) == 'E' || input.charAt(i) == 'F')
	    	                continue;
	    	            
	    	            Integer.parseInt(input.charAt(i)+""); //To catch non integer characters 
	    	        }
	    	    }
	    	    catch(Exception e){
	    	        errorSFX2.play();
	    	        message.setForeground(Color.RED);
	    	        message.setText("NOT HEXADECIMAL NUMBER OR IT'S OUT OF BOUNDS(max 8-DIGIT) ! !");
	    	        return;
                }
	    	   
	    	    bin = calc.hexToBinary(input);
	    	    successSFX2.play();
	    	    message.setForeground(Color.GREEN);
	    	    message.setText("HEXADECIMAL NUMBER TRANSFORMED");
	    	    break;
	    	    
	    	case 'A':
	    	    try{
	    	        if(input.length()>4) throw new Exception();
	    	    }
	    	    catch(Exception e){
	    	        errorSFX1.play();
	    	        message.setForeground(Color.RED);
	    	        message.setText("CHARACTER LIMIT IS 4 ! !");
	    	        return;
                }
	    	   
	    	    bin = calc.asciiToBinary(input);
	    	    successSFX1.play();
	    	    message.setForeground(Color.GREEN);
	    	    message.setText("ASCII CHARACTER TRANSFORMED");
	    	    break;
	    }//switch
	    
	    //Write to output
	    output += " \n Binary      : ";
	    for(int i=bin.length-1; i>=0; i--)
	        output += bin[i];
	    
	    output += " \n Octal       : "+calc.binaryToOctal(bin);
	    output += " \n Decimal     : "+calc.binaryToInteger(bin);
	    output += " \n Hexadecimal : "+calc.binaryToHex(bin);
	    output += " \n ASCII       : "+calc.binaryToAscii(bin);
	    outputText.setText(output);   
	} // organizeInputs()
	
	//Organize Comment Label (Errors and Comments)
	private void organizeComment()
	{
	    message.setForeground(Color.BLUE);
	    
	    switch (currentRadio.charAt(0))
	    {
	    	case 'B':
	    	    message.setText("Enter Binary(0-1) up to 32 bits");
	    	    break;
	    	case 'O':
	    	    message.setText("Enter Octal(Base 8) up to 10 digits");
	    	    break;
	    	case 'D':
	    	    message.setText("Enter Decimal(Base 10-Integer)");
	    	    break;
	    	case 'H':
	    	    message.setText("Enter Hexadecimal(Base 16) up to 8 Hex digits");
	    	    break;
	    	case 'A':
	    	    message.setText("Enter ASCII character up to 4 Characters");
	    	    break;
	    }  
	}//organizeComment()
		
}// End of MainFrame Class

